@@ -17,19 +17,22 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import logging
 from collections import defaultdict
 from functools import partial
-import logging
+
 import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.components import Component, Exploration, PreprocessorStack, Synchronizable, Policy, Optimizer, \
-    ValueFunction, ContainerMerger, ContainerSplitter
+    ContainerMerger, ContainerSplitter
 from rlgraph.graphs.graph_builder import GraphBuilder
 from rlgraph.graphs.graph_executor import GraphExecutor
 from rlgraph.spaces import Space, ContainerSpace
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
-from rlgraph.utils.input_parsing import parse_execution_spec, parse_observe_spec, parse_update_spec
+from rlgraph.utils.input_parsing import parse_execution_spec, parse_observe_spec, parse_update_spec, \
+    parse_value_function_spec
+from rlgraph.utils.ops import flatten_op
 from rlgraph.utils.specifiable import Specifiable
 
 if get_backend() == "tf":
@@ -64,13 +67,14 @@ class Agent(Specifiable):
                 Space object for the Space(s) of the internal (RNN) states.
 
             policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
-            value_function_spec (list, dict): Neural network specification for baseline.
+            value_function_spec (list, dict, ValueFunction): Neural network specification for baseline or instance
+                of ValueFunction.
 
             exploration_spec (Optional[dict]): The spec-dict to create the Exploration Component.
             execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
             optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
 
-            value_function_optimizer_spec (dict): Optimizer config for value function otpimizer. If None, the optimizer
+            value_function_optimizer_spec (dict): Optimizer config for value function optimizer. If None, the optimizer
                 spec for the policy is used (same learning rate and optimizer type).
 
             observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
@@ -95,8 +99,7 @@ class Agent(Specifiable):
             if isinstance(self.state_space, ContainerSpace) else None
         self.logger.info("Parsed state space definition: {}".format(self.state_space))
         self.action_space = Space.from_spec(action_space).with_batch_rank(False)
-        self.flat_action_space = self.action_space.flatten(scope_separator_at_start=False)\
-            if isinstance(self.action_space, ContainerSpace) else None
+        self.flat_action_space = self.action_space.flatten() if isinstance(self.action_space, ContainerSpace) else None
         self.logger.info("Parsed action space definition: {}".format(self.action_space))
 
         self.discount = discount
@@ -123,7 +126,7 @@ class Agent(Specifiable):
             self.logger.info("No preprocessing required.")
 
         # Construct the Policy network.
-        policy_spec = policy_spec or dict()
+        policy_spec = policy_spec or {}
         if "network_spec" not in policy_spec:
             policy_spec["network_spec"] = network_spec
         if "action_space" not in policy_spec:
@@ -135,20 +138,14 @@ class Agent(Specifiable):
         self.policy.add_components(Synchronizable(), expose_apis="sync")
 
         # Create non-shared baseline network.
-        self.value_function = None
+        self.value_function = parse_value_function_spec(value_function_spec)
         # TODO move this to specific agents.
-        if value_function_spec is not None:
-            if isinstance(value_function_spec, list):
-                # Use default type if given is list.
-                value_function_spec = dict(type="value_function", network_spec=value_function_spec)
-            self.value_function = ValueFunction.from_spec(value_function_spec)
-            self.value_function.add_components(Synchronizable(), expose_apis="sync")
+        if self.value_function is not None:
             self.vars_merger = ContainerMerger("policy", "vf", scope="variable-dict-merger")
             self.vars_splitter = ContainerSplitter("policy", "vf", scope="variable-container-splitter")
         else:
             self.vars_merger = ContainerMerger("policy", scope="variable-dict-merger")
             self.vars_splitter = ContainerSplitter("policy", scope="variable-container-splitter")
-
         self.internal_states_space = Space.from_spec(internal_states_space)
 
         # An object implementing the loss function interface is only strictly needed
@@ -410,8 +407,9 @@ class Agent(Specifiable):
                     self.states_buffer[env_id].extend(preprocessed_states)
                     self.next_states_buffer[env_id].extend(next_states)
                 if self.flat_action_space is not None:
+                    flat_action = flatten_op(actions)
                     for i, flat_key in enumerate(self.flat_action_space.keys()):
-                        self.actions_buffer[env_id][i].extend(actions[flat_key])
+                        self.actions_buffer[env_id][i].append(flat_action[flat_key])
                 else:
                     self.actions_buffer[env_id].extend(actions)
                 self.internals_buffer[env_id].extend(internals)
@@ -427,8 +425,9 @@ class Agent(Specifiable):
                     self.states_buffer[env_id].append(preprocessed_states)
                     self.next_states_buffer[env_id].append(next_states)
                 if self.flat_action_space is not None:
+                    flat_action = flatten_op(actions)
                     for i, flat_key in enumerate(self.flat_action_space.keys()):
-                        self.actions_buffer[env_id][i].append(actions[flat_key])
+                        self.actions_buffer[env_id][i].append(flat_action[flat_key])
                 else:
                     self.actions_buffer[env_id].append(actions)
                 self.internals_buffer[env_id].append(internals)
@@ -438,9 +437,16 @@ class Agent(Specifiable):
             buffer_is_full = len(self.rewards_buffer[env_id]) >= self.observe_spec["buffer_size"]
 
             # If the buffer (per environment) is full OR the episode was aborted:
-            # Change terminal of last record artificially to True, insert and flush the buffer.
+            # Change terminal of last record artificially to True (also give warning "buffer too small"),
+            # insert and flush the buffer.
             if buffer_is_full or self.terminals_buffer[env_id][-1]:
-                self.terminals_buffer[env_id][-1] = True
+                # Warn if full and last terminal is False.
+                if buffer_is_full and not self.terminals_buffer[env_id][-1]:
+                    self.logger.warning(
+                        "Buffer of size {} of Agent '{}' may be too small! Had to add artificial terminal=True "
+                        "to end.".format(self.observe_spec["buffer_size"], self)
+                    )
+                    self.terminals_buffer[env_id][-1] = True
 
                 # TODO: Apply n-step post-processing if necessary.
                 if self.flat_action_space is not None:
@@ -629,3 +635,10 @@ class Agent(Specifiable):
             any: Post-processed batch.
         """
         pass
+
+    def __repr__(self):
+        """
+        Returns:
+            str: A short, but informative description for this Agent.
+        """
+        raise NotImplementedError
