@@ -34,14 +34,61 @@ class DQFDAgent(Agent):
 
     https://arxiv.org/abs/1704.03732
     """
-    def __init__(self, expert_margin=0.5, supervised_weight=1.0, double_q=True, dueling_q=True,
-                 huber_loss=False, n_step=1, shared_container_action_target=False,
-                 memory_spec=None, demo_memory_spec=None,
-                 demo_sample_ratio=0.2, store_last_memory_batch=False, store_last_q_table=False, **kwargs):
-        # TODO Most of this is DQN duplicate but the way the loss function is instantiated, inheriting
-        # from DQN does not work well.
+    def __init__(
+        self,
+        state_space,
+        action_space,
+        discount=0.98,
+        preprocessing_spec=None,
+        network_spec=None,
+        internal_states_space=None,
+        policy_spec=None,
+        exploration_spec=None,
+        execution_spec=None,
+        optimizer_spec=None,
+        observe_spec=None,
+        update_spec=None,
+        summary_spec=None,
+        saver_spec=None,
+        auto_build=True,
+        name="dqfd-agent",
+        expert_margin=0.5,
+        supervised_weight=1.0,
+        double_q=True,
+        dueling_q=True,
+        huber_loss=False,
+        n_step=1,
+        shared_container_action_target=False,
+        memory_spec=None,
+        demo_memory_spec=None,
+        demo_sample_ratio=0.2,
+        store_last_memory_batch=False,
+        store_last_q_table=False
+    ):
+
         """
         Args:
+            state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
+            action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+            preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
+                preprocessing steps or a PreprocessorStack object itself.
+            discount (float): The discount factor (gamma).
+            network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
+                object itself.
+            internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
+                Space object for the Space(s) of the internal (RNN) states.
+            policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+            exploration_spec (Optional[dict]): The spec-dict to create the Exploration Component.
+            execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
+            optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+            observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
+            update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
+            summary_spec (Optional[dict]): Spec-dict to specify summary settings.
+            saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+            auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
+                graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
+                components before building.
+            name (str): Some name for this Agent object.
             expert_margin (float): The expert margin enforces a distance in Q-values between expert action and
                 all other actions.
             supervised_weight (float): Indicates weight of the expert loss.
@@ -59,7 +106,6 @@ class DQFDAgent(Agent):
                 Default: False.
         """
         # Fix action-adapter before passing it to the super constructor.
-        policy_spec = kwargs.pop("policy_spec", dict())
         # Use a DuelingPolicy (instead of a basic Policy) if option is set.
         if dueling_q is True:
             policy_spec["type"] = "dueling-policy"
@@ -67,7 +113,22 @@ class DQFDAgent(Agent):
             if "units_state_value_stream" not in policy_spec:
                 policy_spec["units_state_value_stream"] = 128
         super(DQFDAgent, self).__init__(
-            policy_spec=policy_spec, name=kwargs.pop("name", "dqfd-agent"), **kwargs
+            state_space=state_space,
+            action_space=action_space,
+            discount=discount,
+            preprocessing_spec=preprocessing_spec,
+            network_spec=network_spec,
+            internal_states_space=internal_states_space,
+            policy_spec=policy_spec,
+            exploration_spec=exploration_spec,
+            execution_spec=execution_spec,
+            optimizer_spec=optimizer_spec,
+            observe_spec=observe_spec,
+            update_spec=update_spec,
+            summary_spec=summary_spec,
+            saver_spec=saver_spec,
+            auto_build=auto_build,
+            name=name
         )
         # Assert that the synch interval is a multiple of the update_interval.
         if self.update_spec["sync_interval"] / self.update_spec["update_interval"] != \
@@ -211,7 +272,7 @@ class DQFDAgent(Agent):
         def update_from_memory(root, apply_demo_loss, expert_margins):
             # Sample from online memory.
             records, sample_indices, importance_weights = agent.memory.get_records(self.batch_size)
-            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
+            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
 
             # Do not apply demo loss to online experience.
             online_step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
@@ -231,7 +292,7 @@ class DQFDAgent(Agent):
         def update_from_demos(root, demo_batch_size, apply_demo_loss, expert_margins):
             # Sample from demo memory.
             records, sample_indices, importance_weights = agent.demo_memory.get_records(demo_batch_size)
-            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
+            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
             step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
                 expert_margins
@@ -269,12 +330,12 @@ class DQFDAgent(Agent):
             optimizer = root.get_sub_component_by_name(agent.optimizer.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_adapter_outputs(preprocessed_states)["adapter_outputs"]
+            qt_values_sp = target_policy.get_adapter_outputs(preprocessed_next_states)["adapter_outputs"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_adapter_outputs(preprocessed_next_states)["adapter_outputs"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins, q_values_sp,
@@ -301,12 +362,12 @@ class DQFDAgent(Agent):
             loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_adapter_outputs(preprocessed_states)["adapter_outputs"]
+            qt_values_sp = target_policy.get_adapter_outputs(preprocessed_next_states)["adapter_outputs"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_adapter_outputs(preprocessed_next_states)["adapter_outputs"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins,
@@ -497,4 +558,5 @@ class DQFDAgent(Agent):
         self.graph_executor.execute(("insert_demos", [preprocessed_states, actions, rewards, next_states, terminals]))
 
     def __repr__(self):
-        return "DQFDAgent(doubleQ={} duelingQ={})".format(self.double_q, self.dueling_q)
+        return "DQFDAgent(doubleQ={} duelingQ={}, expert margin={})".format(self.double_q, self.dueling_q,
+                                                                            self.expert_margin)
