@@ -36,10 +36,56 @@ class DQNAgent(Agent):
     [3] Dueling Network Architectures for Deep Reinforcement Learning, Wang et al. - 2016
     [4] https://en.wikipedia.org/wiki/Huber_loss
     """
-    def __init__(self, double_q=True, dueling_q=True, huber_loss=False, n_step=1, shared_container_action_target=True,
-                 memory_spec=None, store_last_memory_batch=False, store_last_q_table=False, **kwargs):
+    def __init__(
+        self,
+        state_space,
+        action_space,
+        discount=0.98,
+        preprocessing_spec=None,
+        network_spec=None,
+        internal_states_space=None,
+        policy_spec=None,
+        exploration_spec=None,
+        execution_spec=None,
+        optimizer_spec=None,
+        observe_spec=None,
+        update_spec=None,
+        summary_spec=None,
+        saver_spec=None,
+        auto_build=True,
+        name="dqn-agent",
+        double_q=True,
+        dueling_q=True,
+        huber_loss=False,
+        n_step=1,
+        shared_container_action_target=True,
+        memory_spec=None,
+        store_last_memory_batch=False,
+        store_last_q_table=False,
+    ):
         """
         Args:
+            state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
+            action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+            preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
+                preprocessing steps or a PreprocessorStack object itself.
+            discount (float): The discount factor (gamma).
+            network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
+                object itself.
+            internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
+                Space object for the Space(s) of the internal (RNN) states.
+            policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+            exploration_spec (Optional[dict]): The spec-dict to create the Exploration Component.
+            execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
+            optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+            observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
+            update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
+            summary_spec (Optional[dict]): Spec-dict to specify summary settings.
+            saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+            auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
+                graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
+                components before building.
+            name (str): Some name for this Agent object.
             double_q (bool): Whether to use the double DQN loss function (see [2]).
             dueling_q (bool): Whether to use a dueling layer in the ActionAdapter  (see [3]).
             huber_loss (bool) : Whether to apply a Huber loss. (see [4]).
@@ -53,7 +99,6 @@ class DQNAgent(Agent):
                 Default: False.
         """
         # Fix action-adapter before passing it to the super constructor.
-        policy_spec = kwargs.pop("policy_spec", dict())
         # Use a DuelingPolicy (instead of a basic Policy) if option is set.
         if dueling_q is True:
             policy_spec["type"] = "dueling-policy"
@@ -62,12 +107,27 @@ class DQNAgent(Agent):
                 policy_spec["units_state_value_stream"] = 128
 
         super(DQNAgent, self).__init__(
-            policy_spec=policy_spec, name=kwargs.pop("name", "dqn-agent"), **kwargs
+            state_space=state_space,
+            action_space=action_space,
+            discount=discount,
+            preprocessing_spec=preprocessing_spec,
+            network_spec=network_spec,
+            internal_states_space=internal_states_space,
+            policy_spec=policy_spec,
+            exploration_spec=exploration_spec,
+            execution_spec=execution_spec,
+            optimizer_spec=optimizer_spec,
+            observe_spec=observe_spec,
+            update_spec=update_spec,
+            summary_spec=summary_spec,
+            saver_spec=saver_spec,
+            auto_build=auto_build,
+            name=name
         )
 
         # TODO: Have to manually set it here for multi-GPU synchronizer to know its number
         # TODO: of return values when calling _graph_fn_calculate_update_from_external_batch.
-        #self.root_component.graph_fn_num_outputs["_graph_fn_update_from_external_batch"] = 4
+        # self.root_component.graph_fn_num_outputs["_graph_fn_update_from_external_batch"] = 4
 
         # Assert that the synch interval is a multiple of the update_interval.
         if self.update_spec["sync_interval"] / self.update_spec["update_interval"] != \
@@ -96,7 +156,7 @@ class DQNAgent(Agent):
 
         self.input_spaces.update(dict(
             actions=self.action_space.with_batch_rank(),
-            # weights will have a Space derived from the vars of policy.
+            # Weights will have a Space derived from the vars of policy.
             policy_weights="variables:{}".format(self.policy.scope),
             time_step=int,
             use_exploration=bool,
@@ -200,7 +260,7 @@ class DQNAgent(Agent):
         def update_from_memory(root, apply_postprocessing):
             # Non prioritized memory will just return weight 1.0 for all samples.
             records, sample_indices, importance_weights = agent.memory.get_records(agent.update_spec["batch_size"])
-            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
+            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
 
             step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights,
@@ -229,7 +289,7 @@ class DQNAgent(Agent):
                     all_vars, preprocessed_states, actions, rewards, terminals,
                     preprocessed_next_states, importance_weights, apply_postprocessing=apply_postprocessing
                 )
-                avg_grads_and_vars = agent.vars_splitter.split(out["avg_grads_and_vars_by_component"])
+                avg_grads_and_vars = agent.vars_splitter.call(out["avg_grads_and_vars_by_component"])
                 step_op = agent.optimizer.apply_gradients(avg_grads_and_vars)
                 # Increase the global training step counter.
                 step_op = root._graph_fn_training_step(step_op)
@@ -247,12 +307,12 @@ class DQNAgent(Agent):
             vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_adapter_outputs_and_parameters(preprocessed_states)["adapter_outputs"]
+            qt_values_sp = target_policy.get_adapter_outputs_and_parameters(preprocessed_next_states)["adapter_outputs"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_adapter_outputs_and_parameters(preprocessed_next_states)["adapter_outputs"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights
@@ -283,12 +343,12 @@ class DQNAgent(Agent):
             loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_adapter_outputs_and_parameters(preprocessed_states)["adapter_outputs"]
+            qt_values_sp = target_policy.get_adapter_outputs_and_parameters(preprocessed_next_states)["adapter_outputs"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_adapter_outputs_and_parameters(preprocessed_next_states)["adapter_outputs"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights
