@@ -18,24 +18,25 @@ from __future__ import division
 from __future__ import print_function
 
 import copy
+
 import tensorflow as tf
 
-from rlgraph.utils.decorators import rlgraph_api, graph_fn
-from rlgraph.utils import RLGraphError
 from rlgraph.agents.agent import Agent
 from rlgraph.components.common.container_merger import ContainerMerger
-from rlgraph.components.common.container_splitter import ContainerSplitter
+from rlgraph.components.common.environment_stepper import EnvironmentStepper
 from rlgraph.components.common.slice import Slice
 from rlgraph.components.common.staging_area import StagingArea
-from rlgraph.components.common.environment_stepper import EnvironmentStepper
+from rlgraph.components.layers.preprocessing.container_splitter import ContainerSplitter
 from rlgraph.components.layers.preprocessing.reshape import ReShape
 from rlgraph.components.layers.preprocessing.transpose import Transpose
-from rlgraph.components.neural_networks.actor_component import ActorComponent
-from rlgraph.components.policies.dynamic_batching_policy import DynamicBatchingPolicy
 from rlgraph.components.loss_functions.impala_loss_function import IMPALALossFunction
 from rlgraph.components.memories.fifo_queue import FIFOQueue
 from rlgraph.components.memories.queue_runner import QueueRunner
+from rlgraph.components.neural_networks.actor_component import ActorComponent
+from rlgraph.components.policies.dynamic_batching_policy import DynamicBatchingPolicy
 from rlgraph.spaces import FloatBox, Dict, Tuple
+from rlgraph.utils import RLGraphError
+from rlgraph.utils.decorators import rlgraph_api, graph_fn
 from rlgraph.utils.util import default_dict
 
 
@@ -380,7 +381,7 @@ class IMPALAAgent(Agent):
             # Take n steps in the environment.
             step_results = env_stepper.step()
 
-            split_output = env_output_splitter.split(step_results)
+            split_output = env_output_splitter.call(step_results)
             # Slice off the initial internal state (so the learner can re-feed-forward from that internal-state).
             initial_internal_states = internal_states_slicer.slice(split_output[-1], 0)  # -1=internal states
             to_merge = split_output[:-1] + (initial_internal_states,)
@@ -433,7 +434,7 @@ class IMPALAAgent(Agent):
             # But must still be done for actions, rewards, terminals here in this API-method via separate ReShapers.
             records = fifo_queue.get_records(self.update_spec["batch_size"])
 
-            split_record = fifo_output_splitter.split(records)
+            split_record = fifo_output_splitter.call(records)
             actions = None
             rewards = None
             if self.feed_previous_action_through_nn and self.feed_previous_reward_through_nn:
@@ -444,13 +445,13 @@ class IMPALAAgent(Agent):
             # Flip everything to time-major.
             # TODO: Create components that are less input-space sensitive (those that have no variables should
             # TODO: be reused for any kind of processing)
-            states = transposer.apply(states)
-            terminals = transposer.apply(terminals)
-            action_probs_mu = transposer.apply(action_probs_mu)
+            states = transposer.call(states)
+            terminals = transposer.call(terminals)
+            action_probs_mu = transposer.call(action_probs_mu)
             if self.feed_previous_action_through_nn is False:
-                actions = transposer.apply(actions)
+                actions = transposer.call(actions)
             if self.feed_previous_reward_through_nn is False:
-                rewards = transposer.apply(rewards)
+                rewards = transposer.call(rewards)
 
             # If we use a GPU: Put everything on staging area (adds 1 time step policy lag, but makes copying
             # data into GPU more efficient).
@@ -466,14 +467,14 @@ class IMPALAAgent(Agent):
             preprocessed_states = preprocessor.preprocess(states)
 
             # Only retrieve logits and do faster sparse softmax in loss.
-            out = policy.get_state_values_logits_parameters_log_probs(preprocessed_states, initial_internal_states)
+            out = policy.get_state_values_adapter_outputs_and_parameters(preprocessed_states, initial_internal_states)
             state_values_pi = out["state_values"]
-            logits = out["logits"]
+            logits = out["adapter_outputs"]
             #current_internal_states = out["last_internal_states"]
 
             # Isolate actions and rewards from states.
             if self.feed_previous_action_through_nn or self.feed_previous_reward_through_nn:
-                states_split = states_dict_splitter.split(states)
+                states_split = states_dict_splitter.call(states)
                 actions = states_split[-2]
                 rewards = states_split[-1]
 
@@ -715,15 +716,15 @@ class SingleIMPALAAgent(IMPALAAgent):
             # Flip everything to time-major.
             # TODO: Create components that are less input-space sensitive (those that have no variables should
             # TODO: be reused for any kind of processing: already done, use space_agnostic feature. See ReShape)
-            states = agent.transposer.apply(states)
-            terminals = agent.transposer.apply(terminals)
-            action_probs_mu = agent.transposer.apply(action_probs_mu)
+            states = agent.transposer.call(states)
+            terminals = agent.transposer.call(terminals)
+            action_probs_mu = agent.transposer.call(action_probs_mu)
             actions = None
             if not self.feed_previous_action_through_nn:
-                actions = agent.transposer.apply(out["actions"])
+                actions = agent.transposer.call(out["actions"])
             rewards = None
             if not self.feed_previous_reward_through_nn:
-                rewards = agent.transposer.apply(out["rewards"])
+                rewards = agent.transposer.call(out["rewards"])
 
             # If we use a GPU: Put everything on staging area (adds 1 time step policy lag, but makes copying
             # data into GPU more efficient).
@@ -743,14 +744,14 @@ class SingleIMPALAAgent(IMPALAAgent):
                 states = agent.preprocessor.preprocess(states)
 
             # Get the pi-action probs AND the values for all our states.
-            out = agent.policy.get_state_values_logits_parameters_log_probs(states, initial_internal_states)
+            out = agent.policy.get_state_values_adapter_outputs_and_parameters(states, initial_internal_states)
             state_values_pi = out["state_values"]
-            logits_pi = out["logits"]
+            logits_pi = out["adapter_outputs"]
 
             # Isolate actions and rewards from states.
             # TODO: What if only one of actions or rewards is fed through NN, but the other not?
             if self.feed_previous_reward_through_nn and self.feed_previous_action_through_nn:
-                out = agent.states_dict_splitter.split(states)
+                out = agent.states_dict_splitter.call(states)
                 actions = out[-2]  # TODO: Are these always the correct slots for "previous_action" and "previous_reward"?
                 rewards = out[-1]
 
