@@ -21,14 +21,14 @@ import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.agents import Agent
-from rlgraph.components import ContainerMerger, ContainerSplitter, Memory, RingBuffer, PPOLossFunction
+from rlgraph.components import ContainerMerger, Memory, RingBuffer, PPOLossFunction
 from rlgraph.components.helpers import GeneralizedAdvantageEstimation
 from rlgraph.spaces import BoolBox, FloatBox
 from rlgraph.utils import util
-from rlgraph.utils.execution_util import define_by_run_flatten
-from rlgraph.utils.ops import flatten_op, DataOpDict
-from rlgraph.utils.util import strip_list
 from rlgraph.utils.decorators import rlgraph_api
+from rlgraph.utils.define_by_run_ops import define_by_run_flatten
+from rlgraph.utils.ops import flatten_op, unflatten_op, DataOpDict, ContainerDataOp, FlattenedDataOp
+from rlgraph.utils.util import strip_list
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -45,13 +45,65 @@ class PPOAgent(Agent):
     Paper: https://arxiv.org/abs/1707.06347
     """
 
-    def __init__(self, clip_ratio=0.2, gae_lambda=1.0, clip_rewards=0.0, standardize_advantages=False,
-                 sample_episodes=True, weight_entropy=None, memory_spec=None, **kwargs):
+    def __init__(
+        self,
+        state_space,
+        action_space,
+        discount=0.98,
+        preprocessing_spec=None,
+        network_spec=None,
+        internal_states_space=None,
+        policy_spec=None,
+        value_function_spec=None,
+        execution_spec=None,
+        optimizer_spec=None,
+        value_function_optimizer_spec=None,
+        observe_spec=None,
+        update_spec=None,
+        summary_spec=None,
+        saver_spec=None,
+        auto_build=True,
+        name="ppo-agent",
+        clip_ratio=0.2,
+        gae_lambda=1.0,
+        clip_rewards=0.0,
+        value_function_clipping=None,
+        standardize_advantages=False,
+        sample_episodes=True,
+        weight_entropy=None,
+        memory_spec=None
+    ):
         """
         Args:
+            state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
+            action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+            preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
+                preprocessing steps or a PreprocessorStack object itself.
+            discount (float): The discount factor (gamma).
+            network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
+                object itself.
+            internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
+                Space object for the Space(s) of the internal (RNN) states.
+            policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+            value_function_spec (list, dict, ValueFunction): Neural network specification for baseline or instance
+                of ValueFunction.
+            execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
+            optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+            value_function_optimizer_spec (dict): Optimizer config for value function optimizer. If None, the optimizer
+                spec for the policy is used (same learning rate and optimizer type).
+            observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
+            update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
+            summary_spec (Optional[dict]): Spec-dict to specify summary settings.
+            saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+            auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
+                graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
+                components before building.
+            name (str): Some name for this Agent object.
             clip_ratio (float): Clipping parameter for likelihood ratio.
             gae_lambda (float): Lambda for generalized advantage estimation.
             clip_rewards (float): Reward clip value. If not 0, rewards will be clipped into this range.
+            value_function_clipping (Optional[float]): If not None, uses clipped value function objective. If None,
+                uses simple value function objective.
             standardize_advantages (bool): If true, standardize advantage values in update.
 
             sample_episodes (bool): If True, the update method interprets the batch_size as the number of
@@ -64,14 +116,28 @@ class PPOAgent(Agent):
             memory_spec (Optional[dict,Memory]): The spec for the Memory to use. Should typically be
                 a ring-buffer.
         """
-        if "policy_spec" in kwargs:
-            policy_spec = kwargs.pop("policy_spec")
+        if policy_spec is not None:
             policy_spec["deterministic"] = False
         else:
             policy_spec = dict(deterministic=False)
         super(PPOAgent, self).__init__(
-            policy_spec=policy_spec,  # Set policy to stochastic.
-            name=kwargs.pop("name", "ppo-agent"), **kwargs
+            state_space=state_space,
+            action_space=action_space,
+            discount=discount,
+            preprocessing_spec=preprocessing_spec,
+            network_spec=network_spec,
+            internal_states_space=internal_states_space,
+            policy_spec=policy_spec,
+            value_function_spec=value_function_spec,
+            execution_spec=execution_spec,
+            optimizer_spec=optimizer_spec,
+            value_function_optimizer_spec=value_function_optimizer_spec,
+            observe_spec=observe_spec,
+            update_spec=update_spec,
+            summary_spec=summary_spec,
+            saver_spec=saver_spec,
+            name=name,
+            auto_build=auto_build
         )
         self.sample_episodes = sample_episodes
 
@@ -104,15 +170,16 @@ class PPOAgent(Agent):
         # Make sure the python buffer is not larger than our memory capacity.
         assert self.observe_spec["buffer_size"] <= self.memory.capacity, \
             "ERROR: Buffer's size ({}) in `observe_spec` must be smaller or equal to the memory's capacity ({})!". \
-                format(self.observe_spec["buffer_size"], self.memory.capacity)
+            format(self.observe_spec["buffer_size"], self.memory.capacity)
 
         # The splitter for splitting up the records coming from the memory.
-        self.splitter = ContainerSplitter("states", "actions", "rewards", "terminals")
-        self.gae_function = GeneralizedAdvantageEstimation(gae_lambda=gae_lambda, discount=self.discount,
-                                                           clip_rewards=clip_rewards)
-        self.loss_function = PPOLossFunction(clip_ratio=clip_ratio,
-                                             standardize_advantages=standardize_advantages,
-                                             weight_entropy=weight_entropy)
+        self.standardize_advantages = standardize_advantages
+        self.gae_function = GeneralizedAdvantageEstimation(
+            gae_lambda=gae_lambda, discount=self.discount, clip_rewards=clip_rewards
+        )
+        self.loss_function = PPOLossFunction(
+            clip_ratio=clip_ratio, value_function_clipping=value_function_clipping, weight_entropy=weight_entropy
+        )
 
         self.iterations = self.update_spec["num_iterations"]
         self.sample_size = self.update_spec["sample_size"]
@@ -120,7 +187,8 @@ class PPOAgent(Agent):
 
         # Add all our sub-components to the core.
         self.root_component.add_components(
-            self.preprocessor, self.merger, self.memory, self.splitter, self.policy, self.exploration,
+            self.preprocessor, self.merger, self.memory,
+            self.policy, self.exploration,
             self.loss_function, self.optimizer, self.value_function, self.value_function_optimizer, self.vars_merger,
             self.vars_splitter, self.gae_function
         )
@@ -181,13 +249,14 @@ class PPOAgent(Agent):
                 records = agent.memory.get_episodes(self.update_spec["batch_size"])
             else:
                 records = agent.memory.get_records(self.update_spec["batch_size"])
-            preprocessed_s, actions, rewards, terminals = agent.splitter.split(records)
 
             # Route to post process and update method.
             # Use terminals as sequence indices.
-            sequence_indices = terminals
-            return root.update_from_external_batch(preprocessed_s, actions, rewards, terminals,
-                                                   sequence_indices, apply_postprocessing)
+            sequence_indices = records["terminals"]
+            return root.update_from_external_batch(
+                records["states"], records["actions"], records["rewards"], records["terminals"],
+                sequence_indices, apply_postprocessing
+            )
 
         # N.b. this is here because the iterative_optimization would need policy/losses as sub-components, but
         # multiple parents are not allowed currently.
@@ -210,34 +279,47 @@ class PPOAgent(Agent):
             value_function_optimizer = root.get_sub_component_by_name(agent.value_function_optimizer.scope)
             vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
             gae_function = root.get_sub_component_by_name(agent.gae_function.scope)
-            prev_log_probs = policy.get_action_log_probs(preprocessed_states, actions)["action_log_probs"]
+            prev_log_probs = policy.get_log_likelihood(preprocessed_states, actions)["log_likelihood"]
 
             if get_backend() == "tf":
                 # Log probs before update.
-                if isinstance(prev_log_probs, DataOpDict):
-                    for name in prev_log_probs.keys():
-                        prev_log_probs[name] = tf.stop_gradient(prev_log_probs[name])
-                else:
-                    prev_log_probs = tf.stop_gradient(prev_log_probs)
+                prev_log_probs = tf.stop_gradient(prev_log_probs)
                 batch_size = tf.shape(preprocessed_states)[0]
+                prior_baseline_values = tf.stop_gradient(value_function.value_output(preprocessed_states))
+
+                # Advantages are based on prior baseline values.
+                advantages = tf.cond(
+                        pred=apply_postprocessing,
+                        true_fn=lambda: gae_function.calc_gae_values(
+                            prior_baseline_values, rewards, terminals, sequence_indices),
+                        false_fn=lambda: rewards
+                    )
+
+                if self.standardize_advantages:
+                    mean, std = tf.nn.moments(x=advantages, axes=[0])
+                    advantages = (advantages - mean) / std
 
                 def opt_body(index_, loss_, loss_per_item_, vf_loss_, vf_loss_per_item_):
                     start = tf.random_uniform(shape=(), minval=0, maxval=batch_size - 1, dtype=tf.int32)
                     indices = tf.range(start=start, limit=start + agent.sample_size) % batch_size
                     sample_states = tf.gather(params=preprocessed_states, indices=indices)
-                    if isinstance(actions, dict):
-                        sample_actions = DataOpDict()
-                        sample_prior_log_probs = DataOpDict()
-                        for name, action in flatten_op(actions, scope_separator_at_start=False).items():
+                    if isinstance(actions, ContainerDataOp):
+                        sample_actions = FlattenedDataOp()
+                        for name, action in flatten_op(actions).items():
                             sample_actions[name] = tf.gather(params=action, indices=indices)
-                            sample_prior_log_probs[name] = tf.gather(params=prev_log_probs[name], indices=indices)
+                        sample_actions = unflatten_op(sample_actions)
                     else:
                         sample_actions = tf.gather(params=actions, indices=indices)
-                        sample_prior_log_probs = tf.gather(params=prev_log_probs, indices=indices)
 
+                    sample_prior_log_probs = tf.gather(params=prev_log_probs, indices=indices)
                     sample_rewards = tf.gather(params=rewards, indices=indices)
                     sample_terminals = tf.gather(params=terminals, indices=indices)
                     sample_sequence_indices = tf.gather(params=sequence_indices, indices=indices)
+                    sample_advantages = tf.gather(params=advantages, indices=indices)
+                    sample_advantages.set_shape((self.sample_size, ))
+
+                    sample_baseline_values = value_function.value_output(sample_states)
+                    sample_prior_baseline_values = tf.gather(params=prior_baseline_values, indices=indices)
 
                     # If we are a multi-GPU root:
                     # Simply feeds everything into the multi-GPU sync optimizer's method and return.
@@ -251,7 +333,7 @@ class PPOAgent(Agent):
                             sample_states, sample_actions, sample_rewards, sample_terminals, sample_sequence_indices,
                             apply_postprocessing=apply_postprocessing
                         )
-                        avg_grads_and_vars_policy, avg_grads_and_vars_vf = agent.vars_splitter.split(
+                        avg_grads_and_vars_policy, avg_grads_and_vars_vf = agent.vars_splitter.call(
                             out["avg_grads_and_vars_by_component"]
                         )
                         policy_step_op = agent.optimizer.apply_gradients(avg_grads_and_vars_policy)
@@ -274,7 +356,7 @@ class PPOAgent(Agent):
                                 out["loss"] = root._graph_fn_training_step(out["loss"])
                             return index_ + 1, out["loss"], out["loss_per_item"], loss_vf, loss_per_item_vf
 
-                    policy_probs = policy.get_action_log_probs(sample_states, sample_actions)
+                    policy_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
                     baseline_values = value_function.value_output(tf.stop_gradient(sample_states))
                     sample_rewards = tf.cond(
                         pred=apply_postprocessing,
@@ -287,8 +369,8 @@ class PPOAgent(Agent):
 
                     loss, loss_per_item, vf_loss, vf_loss_per_item = \
                         loss_function.loss(
-                            policy_probs["action_log_probs"], sample_prior_log_probs,
-                            baseline_values, sample_rewards,  entropy
+                            policy_probs, sample_prior_log_probs,
+                            sample_baseline_values, sample_prior_baseline_values, sample_advantages,  entropy
                         )
 
                     if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
@@ -346,6 +428,14 @@ class PPOAgent(Agent):
                     prev_log_probs = prev_log_probs.detach()
                 batch_size = preprocessed_states.shape[0]
                 sample_size = min(batch_size, agent.sample_size)
+                prior_baseline_values = value_function.value_output(preprocessed_states).detach()
+                if apply_postprocessing:
+                    advantages = gae_function.calc_gae_values(
+                        prior_baseline_values, rewards, terminals, sequence_indices)
+                else:
+                    advantages = rewards
+                if self.standardize_advantages:
+                    advantages = (advantages - torch.mean(advantages)) / torch.std(advantages)
 
                 for _ in range(agent.iterations):
                     start = int(torch.rand(1) * (batch_size - 1))
@@ -362,29 +452,22 @@ class PPOAgent(Agent):
                         sample_actions = torch.index_select(actions, 0, indices)
                         sample_prior_log_probs = torch.index_select(prev_log_probs, 0, indices)
 
-                    sample_rewards = torch.index_select(rewards, 0, indices)
-                    sample_terminals = torch.index_select(terminals, 0, indices)
-                    sample_sequence_indices = torch.index_select(sequence_indices, 0, indices)
+                    sample_advantages = torch.index_select(advantages, 0, indices)
+                    sample_prior_baseline_values = torch.index_select(prior_baseline_values, 0, indices)
 
-                    policy_probs = policy.get_action_log_probs(sample_states, sample_actions)
-
-                    baseline_values = value_function.value_output(sample_states)
-                    if apply_postprocessing:
-                        sample_rewards = gae_function.calc_gae_values(
-                            baseline_values, sample_rewards, sample_terminals, sample_sequence_indices)
+                    policy_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
+                    sample_baseline_values = value_function.value_output(sample_states)
 
                     entropy = policy.get_entropy(sample_states)["entropy"]
                     loss, loss_per_item, vf_loss, vf_loss_per_item = loss_function.loss(
-                        policy_probs["action_log_probs"], sample_prior_log_probs,
-                        baseline_values,  sample_rewards, entropy
+                        policy_probs, sample_prior_log_probs,
+                        sample_baseline_values,  sample_prior_baseline_values, sample_advantages, entropy
                     )
 
                     # Do not need step op.
                     _, loss, loss_per_item = optimizer.step(policy.variables(), loss, loss_per_item)
-
                     _, vf_loss, vf_loss_per_item = \
                         value_function_optimizer.step(value_function.variables(), vf_loss, vf_loss_per_item)
-
                 return loss, loss_per_item, vf_loss, vf_loss_per_item
 
     def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None):
