@@ -21,16 +21,14 @@ import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.agents import Agent
+from rlgraph.components import Component, Synchronizable, Memory, ValueFunction, ContainerMerger, PrioritizedReplay
+from rlgraph.components.loss_functions.sac_loss_function import SACLossFunction
+from rlgraph.spaces import FloatBox, BoolBox, IntBox, ContainerSpace
 from rlgraph.spaces.space_utils import sanity_check_space
 from rlgraph.utils import RLGraphError
-from rlgraph.spaces import FloatBox, BoolBox, IntBox, Dict
-from rlgraph.components import Component, Synchronizable
-from rlgraph.components.loss_functions.sac_loss_function import SACLossFunction
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
-from rlgraph.components import Memory, ContainerMerger, PrioritizedReplay
-from rlgraph.utils.util import strip_list
 from rlgraph.utils.ops import flatten_op, DataOpTuple
-
+from rlgraph.utils.util import strip_list, force_list
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -106,7 +104,7 @@ class SACAgentComponent(Component):
         for s in ["states", "actions", "env_actions", "preprocessed_states", "rewards", "terminals"]:
             sanity_check_space(input_spaces[s], must_have_batch_rank=True)
 
-        self.env_action_space = input_spaces["env_actions"]
+        self.env_action_space = input_spaces["env_actions"].flatten()
 
     def create_variables(self, input_spaces, action_space=None):
         self.steps_since_last_sync = self.get_variable("steps_since_last_sync", dtype="int", initializer=0)
@@ -128,7 +126,7 @@ class SACAgentComponent(Component):
     """ TODO: need to define the input space
     @rlgraph_api(must_be_complete=False)
     def set_q_weights(self, q_weights):
-        split_weights = self._q_vars_splitter.split(q_weights)
+        split_weights = self._q_vars_splitter.call(q_weights)
         assert len(split_weights) == len(self._q_functions)
         update_ops = [q.sync(q_weights) for q_weights, q in zip(split_weights, self._q_functions)]
         update_ops.extend([q.sync(q_weights) for q_weights, q in zip(split_weights, self._target_q_functions)])
@@ -199,16 +197,10 @@ class SACAgentComponent(Component):
             alpha_loss_per_item=alpha_loss_per_item
         )
 
-    @graph_fn
-    def _graph_fn_one_hot(self, env_actions):
-        # TODO move to util
-        # Checking this separately because env actions will be a raw tensor in define-by-run.
-        if isinstance(self.env_action_space, Dict):
-            for name, space in self.env_action_space.flatten(scope_separator_at_start=False).items():
-                if isinstance(space, IntBox):
-                    env_actions[name] = tf.one_hot(env_actions[name], depth=space.num_categories, axis=-1)
-        elif isinstance(self.env_action_space, IntBox):
-            return tf.one_hot(env_actions, depth=self.env_action_space.num_categories, axis=-1)
+    @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
+    def _graph_fn_one_hot(self, key, env_actions):
+        if isinstance(self.env_action_space[key], IntBox):
+            env_actions = tf.one_hot(env_actions, depth=self.env_action_space[key].num_categories, axis=-1)
         return env_actions
 
     @graph_fn(requires_variable_completeness=True)
@@ -240,17 +232,17 @@ class SACAgentComponent(Component):
     @rlgraph_api
     def get_losses(self, preprocessed_states, actions, rewards, terminals, next_states, importance_weights):
         # TODO: internal states
-        samples_next = self._policy.get_action_and_log_prob(next_states, deterministic=False)
+        samples_next = self._policy.get_action_and_log_likelihood(next_states, deterministic=False)
         next_sampled_actions = samples_next["action"]
-        log_probs_next_sampled = samples_next["log_prob"]
+        log_probs_next_sampled = samples_next["log_likelihood"]
 
         q_values_next_sampled = self.get_q_values(
             next_states, next_sampled_actions, target=True
         )
         q_values = self.get_q_values(preprocessed_states, actions)
-        samples = self._policy.get_action_and_log_prob(preprocessed_states, deterministic=False)
+        samples = self._policy.get_action_and_log_likelihood(preprocessed_states, deterministic=False)
         sampled_actions = samples["action"]
-        log_probs_sampled = samples["log_prob"]
+        log_probs_sampled = samples["log_likelihood"]
         q_values_sampled = self.get_q_values(preprocessed_states, sampled_actions)
 
         alpha = self._graph_fn_compute_alpha()
@@ -362,14 +354,62 @@ class SACAgentComponent(Component):
 
 
 class SACAgent(Agent):
-    def __init__(self, double_q=True, initial_alpha=1.0, gumbel_softmax_temperature=1.0, target_entropy=None,
-                 memory_spec=None, value_function_sync_spec=None, **kwargs):
+    def __init__(
+        self,
+        state_space,
+        action_space,
+        discount=0.98,
+        preprocessing_spec=None,
+        network_spec=None,
+        internal_states_space=None,
+        policy_spec=None,
+        value_function_spec=None,
+        execution_spec=None,
+        optimizer_spec=None,
+        value_function_optimizer_spec=None,
+        observe_spec=None,
+        update_spec=None,
+        summary_spec=None,
+        saver_spec=None,
+        auto_build=True,
+        name="sac-agent",
+        double_q=True,
+        initial_alpha=1.0,
+        gumbel_softmax_temperature=1.0,
+        target_entropy=None,
+        memory_spec=None,
+        value_function_sync_spec=None
+    ):
         """
         This is an implementation of the Soft-Actor Critic algorithm.
 
         Paper: http://arxiv.org/abs/1801.01290
 
         Args:
+            state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
+            action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+            preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
+                preprocessing steps or a PreprocessorStack object itself.
+            discount (float): The discount factor (gamma).
+            network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
+                object itself.
+            internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
+                Space object for the Space(s) of the internal (RNN) states.
+            policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+            value_function_spec (list, dict, ValueFunction): Neural network specification for baseline or instance
+                of ValueFunction.
+            execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
+            optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+            value_function_optimizer_spec (dict): Optimizer config for value function optimizer. If None, the optimizer
+                spec for the policy is used (same learning rate and optimizer type).
+            observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
+            update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
+            summary_spec (Optional[dict]): Spec-dict to specify summary settings.
+            saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+            auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
+                graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
+                components before building.
+            name (str): Some name for this Agent object.
             double_q (bool): Whether to train two q networks independently.
             initial_alpha (float): "The temperature parameter α determines the
                 relative importance of the entropy term against the reward".
@@ -378,20 +418,42 @@ class SACAgent(Agent):
             memory_spec (Optional[dict,Memory]): The spec for the Memory to use for the DQN algorithm.
             update_spec (dict): Here we can have sync_interval or sync_tau (for the value network update).
         """
-        value_function_spec = kwargs.pop("value_function_spec")
-        value_function_spec = dict(type="sac_value_function", network_spec=value_function_spec)
-        super(SACAgent, self).__init__(
+        # If VF spec is a network spec, wrap with SAC vf type. The VF must concatenate actions and states,
+        # which can require splitting the network in the case of e.g. conv-inputs.
+        if isinstance(value_function_spec, list):
+            value_function_spec = dict(type="sac_value_function", network_spec=value_function_spec)
+            self.logger.info("Using default SAC value function.")
+        elif isinstance(value_function_spec, ValueFunction):
+            self.logger.info("Using value function object {}".format(ValueFunction))
+
+        if policy_spec is None:
             # Continuous action space: Use squashed normal.
             # Discrete: Gumbel-softmax.
-            policy_spec=dict(deterministic=False,
-                             distributions_spec=dict(
+            policy_spec = dict(deterministic=False,
+                            distributions_spec=dict(
                                 bounded_distribution_type="squashed",
                                 discrete_distribution_type="gumbel_softmax",
                                 gumbel_softmax_temperature=gumbel_softmax_temperature
-                             )),
-            name=kwargs.pop("name", "sac-agent"),
+                            ))
+
+        super(SACAgent, self).__init__(
+            state_space=state_space,
+            action_space=action_space,
+            discount=discount,
+            preprocessing_spec=preprocessing_spec,
+            network_spec=network_spec,
+            internal_states_space=internal_states_space,
+            policy_spec=policy_spec,
             value_function_spec=value_function_spec,
-            **kwargs
+            execution_spec=execution_spec,
+            optimizer_spec=optimizer_spec,
+            value_function_optimizer_spec=value_function_optimizer_spec,
+            observe_spec=observe_spec,
+            update_spec=update_spec,
+            summary_spec=summary_spec,
+            saver_spec=saver_spec,
+            auto_build=auto_build,
+            name=name
         )
 
         self.double_q = double_q
@@ -421,14 +483,10 @@ class SACAgent(Agent):
 
         self.iterations = self.update_spec["num_iterations"]
         self.batch_size = self.update_spec["batch_size"]
-        float_action_space = self.action_space.with_batch_rank()
 
-        if isinstance(self.action_space, Dict):
-            for name, space in float_action_space.flatten(scope_separator_at_start=False).items():
-                if isinstance(space, IntBox):
-                    float_action_space[name] = space.as_one_hot_float_space()
-        elif isinstance(self.action_space, IntBox):
-            float_action_space = float_action_space.as_one_hot_float_space()
+        float_action_space = self.action_space.with_batch_rank().map(
+            mapping=lambda flat_key, space: space.as_one_hot_float_space() if isinstance(space, IntBox) else space
+        )
 
         self.input_spaces.update(dict(
             env_actions=self.action_space.with_batch_rank(),
@@ -521,28 +579,28 @@ class SACAgent(Agent):
 
         # Control, which return value to "pull" (depending on `additional_returns`).
         return_ops = [0, 1] if "preprocessed_states" in extra_returns else [0]
-        ret = self.graph_executor.execute((
+        ret = force_list(self.graph_executor.execute((
             call_method,
             [batched_states, not use_exploration],  # deterministic = not use_exploration
             # 0=preprocessed_states, 1=action
             return_ops
-        ))
-        # We have a discrete action space -> Convert Gumble (relaxed one-hot) sample back into int type.
-        if isinstance(self.action_space, Dict):
-            actions = ret[0] if "preprocessed_states" in extra_returns else ret
-            for name, space in self.action_space.flatten(scope_separator_at_start=False).items():
-                if isinstance(space, IntBox):
-                    actions[name] = np.array([np.argmax(actions[name][0], axis=-1).astype(space.dtype)])
+        )))
+        # Convert Gumble (relaxed one-hot) sample back into int type for all discrete composite actions.
+        if isinstance(self.action_space, ContainerSpace):
+            ret[0] = ret[0].map(
+                mapping=lambda key, action: np.argmax(action, axis=-1).astype(action.dtype)
+                if isinstance(self.flat_action_space[key], IntBox) else action
+            )
         elif isinstance(self.action_space, IntBox):
-            if "preprocessed_states" in extra_returns:
-                ret = (np.argmax(ret[0], axis=-1).astype(self.action_space.dtype), ret[1])
-            else:
-                ret = np.argmax(ret, axis=-1).astype(self.action_space.dtype)
+            ret[0] = np.argmax(ret[0], axis=-1).astype(self.action_space.dtype)
 
         if remove_batch_rank:
-            return strip_list(ret)
+            ret[0] = strip_list(ret[0])
+
+        if "preprocessed_states" in extra_returns:
+            return ret[0], ret[1]
         else:
-            return ret
+            return ret[0]
 
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
         self.graph_executor.execute((self.root_component.insert_records, [preprocessed_states, actions, rewards, next_states, terminals]))
