@@ -18,12 +18,12 @@ from __future__ import division
 from __future__ import print_function
 
 from rlgraph import get_backend
-from rlgraph.utils.util import convert_dtype
-from rlgraph.utils.decorators import rlgraph_api, graph_fn
 from rlgraph.components.distributions.distribution import Distribution
+from rlgraph.utils.decorators import rlgraph_api, graph_fn
+from rlgraph.utils.util import convert_dtype
 
 if get_backend() == "tf":
-    import tensorflow as tf
+    import tensorflow_probability as tfp
 elif get_backend() == "pytorch":
     import torch
 
@@ -42,11 +42,10 @@ class Bernoulli(Distribution):
             parameters (DataOp): The p value (probability that distribution returns True).
         """
         if get_backend() == "tf":
-            return tf.distributions.Bernoulli(probs=parameters, dtype=convert_dtype("bool"))
+            return tfp.distributions.Bernoulli(probs=parameters, dtype=convert_dtype("bool"))
         elif get_backend() == "pytorch":
             return torch.distributions.Bernoulli(probs=parameters)
 
     @graph_fn
     def _graph_fn_sample_deterministic(self, distribution):
         return distribution.prob(True) >= 0.5
-
