@@ -18,9 +18,8 @@ from __future__ import division
 from __future__ import print_function
 
 from rlgraph import get_backend
-from rlgraph.utils.util import force_list
-
 from rlgraph.components.layers.preprocessing.preprocess_layer import PreprocessLayer
+from rlgraph.utils.util import force_list
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -40,8 +39,7 @@ class Concat(PreprocessLayer):
                 All api_methods to this layer must have the same shape, except for the `axis` rank.
                 Default: -1.
         """
-        super(Concat, self).__init__(flatten_ops=True, split_ops=True,
-                                     scope=scope, **kwargs)
+        super(Concat, self).__init__(flatten_ops=True, split_ops=True, scope=scope, **kwargs)
         self.axis = axis
 
         # Whether input spaces are time-major or not.
@@ -70,9 +68,9 @@ class Concat(PreprocessLayer):
         #        "batch/time-ranks).".format(self.in_space_0.shape, idx, input_spaces[key].shape)
         #    idx += 1
 
-    def _graph_fn_apply(self, *inputs):
+    def _graph_fn_call(self, *inputs):
         if get_backend() == "tf":
-            concat_output = self.layer.apply(force_list(inputs))
+            concat_output = self.layer.call(force_list(inputs))
             # Add batch/time-rank information.
             concat_output._batch_rank = 0 if self.time_major is False else 1
             if self.in_space_0.has_time_rank:
