@@ -20,7 +20,7 @@ from __future__ import print_function
 import numpy as np
 
 from rlgraph import get_backend
-from rlgraph.components.layers.preprocessing import PreprocessLayer
+from rlgraph.components.layers.preprocessing.preprocess_layer import PreprocessLayer
 from rlgraph.spaces import IntBox, FloatBox
 from rlgraph.spaces.space_utils import sanity_check_space, get_space_from_op
 from rlgraph.utils import pytorch_one_hot
@@ -41,7 +41,7 @@ class ReShape(PreprocessLayer):
     (including IntBox categories).
     """
     def __init__(self, new_shape=None, flatten=False, flatten_categories=None, fold_time_rank=False,
-                 unfold_time_rank=False, time_major=None, scope="reshape", **kwargs):
+                 unfold_time_rank=False, time_major=None, scope=None, **kwargs):
         """
         Args:
             new_shape (Optional[Dict[str,Tuple[int]],Tuple[int]]): A dict of str/tuples or a single tuple
@@ -63,20 +63,24 @@ class ReShape(PreprocessLayer):
                 E.g. from (None, None, 2, 3) to (None, 2, 3). Providing both `fold_time_rank` (True) and
                 `new_shape` is allowed.
 
-            unfold_time_rank (bool): Whether to unfold the time rank from a currently common batch+time-rank.
-                The exact size of the time rank to unfold is determined automatically via the original sample.
-                Providing both `unfold_time_rank` (True) and `new_shape` is
-                allowed.
+            unfold_time_rank (Union[bool,int]): Whether to unfold the time rank from a currently common batch+time-rank.
+                The exact size of the time rank to unfold is either directly provided or determined automatically via
+                the original sample.
+                Providing both `unfold_time_rank` (True) and `new_shape` is allowed.
 
             time_major (Optional[bool]): Only used if not None and if unfold_time_rank is True. Specifies whether the
                 time rank should come before the batch rank after unfolding.
         """
+        scope = scope or (
+            "reshape-fold" if fold_time_rank else "reshape-unfold" if unfold_time_rank else
+            "reshape-flatten" if flatten is True else "reshape"
+        )
         super(ReShape, self).__init__(space_agnostic=True, scope=scope, **kwargs)
 
         assert flatten is False or new_shape is None, "ERROR: If `flatten` is True, `new_shape` must be None!"
-        assert fold_time_rank is False or unfold_time_rank is False,\
-            "ERROR: Can only either fold or unfold the time-rank! Both `fold_time_rank` and `unfold_time_rank` cannot " \
-            "be True at the same time."
+        assert not fold_time_rank or not unfold_time_rank,\
+            "ERROR: Can only either fold or unfold the time-rank! Both `fold_time_rank` and `unfold_time_rank` " \
+            "cannot be True at the same time."
 
         # The new shape specifications.
         self.new_shape = new_shape
@@ -111,7 +115,7 @@ class ReShape(PreprocessLayer):
                     add_batch_rank=True, add_time_rank=False
                 )
             # Time rank should be unfolded from batch rank with the given dimension.
-            elif self.unfold_time_rank is True:
+            elif self.unfold_time_rank:
                 sanity_check_space(single_space, must_have_batch_rank=True, must_have_time_rank=False)
                 ret[key] = class_(
                     shape=single_space.shape if new_shape is None else new_shape,
@@ -139,64 +143,70 @@ class ReShape(PreprocessLayer):
         return num_categories
 
     @rlgraph_api(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
-    def _graph_fn_apply(self, key, preprocessing_inputs, input_before_time_rank_folding=None):
+    def _graph_fn_call(self, key, inputs, input_before_time_rank_folding=None):
         """
         Reshapes the input to the specified new shape.
 
         Args:
-            preprocessing_inputs (SingleDataOp): The input to reshape.
+            inputs (SingleDataOp): The input to reshape.
             input_before_time_rank_folding (Optional[SingleDataOp]): The original input (before!) the time-rank had
-                been folded (this was done in a different ReShape Component). Serves if `self.unfold_time_rank` is True
-                to figure out the exact time-rank dimension to unfold.
+                been folded (this was done in a different ReShape Component).
+                Used to figure out the exact time-rank dimension to unfold iff `self.unfold_time_rank` is True.
 
         Returns:
             SingleDataOp: The reshaped input.
         """
-        assert self.unfold_time_rank is False or input_before_time_rank_folding is not None
+        assert self.unfold_time_rank is not True or input_before_time_rank_folding is not None
 
         if self.backend == "python" or get_backend() == "python":
             # Create a one-hot axis for the categories at the end?
-            num_categories = self.get_num_categories(key, get_space_from_op(preprocessing_inputs))
+            num_categories = self.get_num_categories(key, get_space_from_op(inputs))
             if num_categories and num_categories > 1:
-                preprocessing_inputs = one_hot(preprocessing_inputs, depth=num_categories)
+                inputs = one_hot(inputs, depth=num_categories)
 
             if self.unfold_time_rank:
-                new_shape = (-1, -1) + preprocessing_inputs.shape[1:]
+                new_shape = [-1, -1] + list(inputs.shape[1:])
+                if type(self.unfold_time_rank) == int:
+                    new_shape[0 if self.time_major else 1] = self.unfold_time_rank
+                new_shape = tuple(new_shape)
             elif self.fold_time_rank:
-                new_shape = (-1,) + preprocessing_inputs.shape[2:]
+                new_shape = (-1,) + inputs.shape[2:]
             else:
-                new_shape = self.get_preprocessed_space(get_space_from_op(preprocessing_inputs)).get_shape(
+                new_shape = self.get_preprocessed_space(get_space_from_op(inputs)).get_shape(
                     with_batch_rank=-1, with_time_rank=-1
                 )
 
             # Dynamic new shape inference:
             # If both batch and time rank must be left alone OR the time rank must be unfolded from a currently common
             # batch+time 0th rank, get these two dynamically.
-            if len(preprocessing_inputs.shape) > 2 and new_shape[0] == -1 and new_shape[1] == -1:
+            if len(inputs.shape) > 2 and new_shape[0] == -1 and new_shape[1] == -1:
                 # Time rank unfolding. Get the time rank from original input.
                 if self.unfold_time_rank is True:
                     original_shape = input_before_time_rank_folding.shape
                     new_shape = (original_shape[0], original_shape[1]) + new_shape[2:]
                 # No time-rank unfolding, but we do have both batch- and time-rank.
                 else:
-                    input_shape = preprocessing_inputs.shape
+                    input_shape = inputs.shape
                     # Batch and time rank stay as is.
                     new_shape = (input_shape[0], input_shape[1]) + new_shape[2:]
 
-            return np.reshape(preprocessing_inputs, newshape=new_shape)
+            return np.reshape(inputs, newshape=new_shape)
 
         elif get_backend() == "pytorch":
             # Create a one-hot axis for the categories at the end?
-            num_categories = self.get_num_categories(key, get_space_from_op(preprocessing_inputs))
+            num_categories = self.get_num_categories(key, get_space_from_op(inputs))
             if num_categories and num_categories > 1:
-                preprocessing_inputs = pytorch_one_hot(preprocessing_inputs, depth=num_categories)
+                inputs = pytorch_one_hot(inputs, depth=num_categories)
 
             if self.unfold_time_rank:
-                new_shape = (-1, -1) + preprocessing_inputs.shape[1:]
+                new_shape = [-1, -1] + list(inputs.shape[1:])
+                if type(self.unfold_time_rank) == int:
+                    new_shape[0 if self.time_major else 1] = self.unfold_time_rank
+                new_shape = tuple(new_shape)
             elif self.fold_time_rank:
-                new_shape = (-1,) + preprocessing_inputs.shape[2:]
+                new_shape = (-1,) + inputs.shape[2:]
             else:
-                new_shape = self.get_preprocessed_space(get_space_from_op(preprocessing_inputs)).get_shape(
+                new_shape = self.get_preprocessed_space(get_space_from_op(inputs)).get_shape(
                     with_batch_rank=-1, with_time_rank=-1
                 )
 
@@ -210,60 +220,58 @@ class ReShape(PreprocessLayer):
                     new_shape = (original_shape[0], original_shape[1]) + new_shape[2:]
                 # No time-rank unfolding, but we do have both batch- and time-rank.
                 else:
-                    input_shape = preprocessing_inputs.shape
+                    input_shape = inputs.shape
                     # Batch and time rank stay as is.
                     new_shape = (input_shape[0], input_shape[1]) + new_shape[2:]
 
-            # print("Reshaping input of shape {} to new shape {} (flatten = {})".format(preprocessing_inputs.shape,
+            # print("Reshaping input of shape {} to new shape {} (flatten = {})".format(inputs.shape,
             #                                                                           new_shape, self.flatten))
 
-            old_size = np.prod(list(preprocessing_inputs.shape))
+            old_size = np.prod(list(inputs.shape))
             new_size = np.prod(new_shape)
 
             # The problem here is the following: Input has dim e.g. [4, 256, 1, 1]
             # -> If shape inference in spaces failed, output dim is not correct -> reshape will attempt
             # something like reshaping to [256].
-            if self.flatten and preprocessing_inputs.dim() > 1:
-                flattened_shape_without_batchrank = np.prod(preprocessing_inputs.shape[1:])
-                flattened_shape = (preprocessing_inputs.shape[0],) + (flattened_shape_without_batchrank,)
-                return torch.reshape(preprocessing_inputs, flattened_shape)
+            if self.flatten and inputs.dim() > 1:
+                flattened_shape_without_batchrank = np.prod(inputs.shape[1:])
+                flattened_shape = (inputs.shape[0],) + (flattened_shape_without_batchrank,)
+                return torch.reshape(inputs, flattened_shape)
             # If new shape does not fit into old shape, batch inference failed -> try to restore:
             # Equal except batch rank -> return as is:
             elif old_size != new_size:
-                if tuple(preprocessing_inputs.shape[1:]) == new_shape:
-                    return preprocessing_inputs
+                if tuple(inputs.shape[1:]) == new_shape:
+                    return inputs
                 else:
                     # Attempt to rescue reshape by combining new shape with batch dim.
-                    full_new_shape = (preprocessing_inputs.shape[0],) + new_shape
-                    return torch.reshape(preprocessing_inputs, full_new_shape)
+                    full_new_shape = (inputs.shape[0],) + new_shape
+                    return torch.reshape(inputs, full_new_shape)
             else:
-                return torch.reshape(preprocessing_inputs, new_shape)
+                return torch.reshape(inputs, new_shape)
 
         elif get_backend() == "tf":
             # Create a one-hot axis for the categories at the end?
-            space = get_space_from_op(preprocessing_inputs)
+            space = get_space_from_op(inputs)
             num_categories = self.get_num_categories(key, space)
             if num_categories and num_categories > 1:
-                preprocessing_inputs_ = tf.one_hot(
-                    preprocessing_inputs, depth=num_categories, axis=-1, dtype="float32"
+                inputs_ = tf.one_hot(
+                    inputs, depth=num_categories, axis=-1, dtype="float32"
                 )
-                if hasattr(preprocessing_inputs, "_batch_rank"):
-                    preprocessing_inputs_._batch_rank = preprocessing_inputs._batch_rank
-                if hasattr(preprocessing_inputs, "_time_rank"):
-                    preprocessing_inputs_._time_rank = preprocessing_inputs._time_rank
-                preprocessing_inputs = preprocessing_inputs_
+                if hasattr(inputs, "_batch_rank"):
+                    inputs_._batch_rank = inputs._batch_rank
+                if hasattr(inputs, "_time_rank"):
+                    inputs_._time_rank = inputs._time_rank
+                inputs = inputs_
 
-            if self.unfold_time_rank:
-                list_shape = preprocessing_inputs.shape.as_list()
-                assert len(list_shape) == 1 or list_shape[1] is not None,\
-                    "ERROR: Cannot unfold. `preprocessing_inputs` (with shape {}) " \
-                    "already seems to be unfolded!".format(list_shape)
-                new_shape = (-1, -1) + tuple(list_shape[1:])
-            elif self.fold_time_rank:
-                new_shape = (-1,) + tuple(preprocessing_inputs.shape.as_list()[2:])
+            if self.fold_time_rank:
+                new_shape = (-1,) + tuple(inputs.shape.as_list()[2:])
             else:
-                new_shape = self.get_preprocessed_space(get_space_from_op(preprocessing_inputs)).get_shape(
-                    with_batch_rank=-1, with_time_rank=-1
+                time_rank = -1
+                if type(self.unfold_time_rank) == int:
+                    time_rank = self.unfold_time_rank
+
+                new_shape = self.get_preprocessed_space(get_space_from_op(inputs)).get_shape(
+                    with_batch_rank=-1, with_time_rank=time_rank
                 )
 
             # Dynamic new shape inference:
@@ -276,19 +284,17 @@ class ReShape(PreprocessLayer):
                     new_shape = (original_shape[0], original_shape[1]) + new_shape[2:]
                 # No time-rank unfolding, but we do have both batch- and time-rank.
                 else:
-                    input_shape = tf.shape(preprocessing_inputs)
+                    input_shape = tf.shape(inputs)
                     # Batch and time rank stay as is.
                     new_shape = (input_shape[0], input_shape[1]) + new_shape[2:]
 
-            reshaped = tf.reshape(tensor=preprocessing_inputs, shape=new_shape, name="reshaped")
+            reshaped = tf.reshape(tensor=inputs, shape=new_shape, name="reshaped")
 
             # Have to place the time rank back in as unknown (for the auto Space inference).
             if type(self.unfold_time_rank) == int:
-                reshaped._batch_rank = 0
-                reshaped._time_rank = 1
+                reshaped._batch_rank = 1 if self.time_major is True else 0
+                reshaped._time_rank = 0 if self.time_major is True else 1
                 return reshaped
-                # TODO: replace placeholder with default value by _batch_rank/_time_rank properties.
-                #return tf.placeholder_with_default(reshaped, shape=(None, None) + new_shape[2:])
             else:
                 # TODO: add other cases of reshaping and fix batch/time rank hints.
                 if self.fold_time_rank:
