@@ -17,15 +17,17 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from copy import deepcopy
-import numpy as np
-from rlgraph.components import PreprocessorStack
-from six.moves import xrange as range_
 import time
+from copy import deepcopy
 
+import numpy as np
+from six.moves import xrange as range_
+
+from rlgraph.components import PreprocessorStack
+from rlgraph.execution.worker import Worker
+from rlgraph.spaces.containers import Dict, Tuple
 from rlgraph.utils.rlgraph_errors import RLGraphError
 from rlgraph.utils.util import default_dict
-from rlgraph.execution.worker import Worker
 
 
 class SingleThreadedWorker(Worker):
@@ -94,7 +96,7 @@ class SingleThreadedWorker(Worker):
             build_space = in_space
             for sub_comp_scope in scopes:
                 processor_stack.sub_components[sub_comp_scope].create_variables(input_spaces=dict(
-                    preprocessing_inputs=build_space
+                    inputs=build_space
                 ), action_space=None)
                 build_space = processor_stack.sub_components[sub_comp_scope].get_preprocessed_space(build_space)
             processor_stack.reset()
@@ -198,7 +200,6 @@ class SingleThreadedWorker(Worker):
         env_states = self.env_states
         while not (0 < num_timesteps <= timesteps_executed):
             if self.render:
-                # This renders the first underlying environment.
                 self.vector_env.render()
 
             if self.worker_executes_preprocessing:
@@ -225,23 +226,32 @@ class SingleThreadedWorker(Worker):
             # Accumulate the reward over n env-steps (equals one action pick). n=self.frameskip.
             env_rewards = [0 for _ in range_(self.num_environments)]
             next_states = None
-            # For container action spaces, we have to treat each key as an array with batch-rank at index 0.
+
+            # For Dict action spaces, we have to treat each key as an array with batch-rank at index 0.
             # The action-dict is then translated into a list of dicts where each dict contains the original data
             # but without the batch-rank.
             # E.g. {'A': array([0, 1]), 'B': array([2, 3])} -> [{'A': 0, 'B': 2}, {'A': 1, 'B': 3}]
-            if self.agent.flat_action_space is not None:
+            if isinstance(self.agent.action_space, Dict):
                 some_key = next(iter(actions))
                 assert isinstance(actions, dict) and isinstance(actions[some_key], np.ndarray),\
-                    "ERROR: Cannot flip container-action batch with dict keys if returned value is not a dict OR " \
+                    "ERROR: Cannot flip Dict-action batch with dict keys if returned value is not a dict OR " \
                     "values of returned value are not np.ndarrays!"
                 # TODO: What if actions come as nested dicts (more than one level deep)?
-                if hasattr(actions[some_key], "len"):
+                # TODO: Use DataOpDict/Tuple's new `map` method.
+                if hasattr(actions[some_key], "__len__"):
                     env_actions = [{key: value[i] for key, value in actions.items()} for i in range(len(actions[some_key]))]
                 else:
                     # Action was not array type.
                     env_actions = [{key: value for key, value in actions.items()}]
-
-            # No flipping necessary.
+            # Tuple action Spaces:
+            # E.g. Tuple(array([0, 1]), array([2, 3])) -> [(0, 2), (1, 3)]
+            elif isinstance(self.agent.action_space, Tuple):
+                assert isinstance(actions, tuple) and isinstance(actions[0], np.ndarray),\
+                    "ERROR: Cannot flip tuple-action batch if returned value is not a tuple OR " \
+                    "values of returned value are not np.ndarrays!"
+                # TODO: Use DataOpDict/Tuple's new `map` method.
+                env_actions = [tuple(value[i] for _, value in enumerate(actions)) for i in range(len(actions[0]))]
+            # No container batch-flipping necessary.
             else:
                 env_actions = actions
                 if self.num_environments == 1 and env_actions.shape == ():
@@ -278,7 +288,7 @@ class SingleThreadedWorker(Worker):
                     self.finished_episode_timesteps[i].append(self.episode_timesteps[i])
 
                     self.log_finished_episode(
-                        reward=self.episode_returns[i],
+                        episode_return=self.episode_returns[i],
                         duration=episode_duration,
                         timesteps=self.episode_timesteps[i],
                         env_num=i
