@@ -17,26 +17,27 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import numpy as np
 import logging
+
+import numpy as np
 from six.moves import xrange as range_
 
+from rlgraph.environments import VectorEnv, SequentialVectorEnv
 from rlgraph.utils.specifiable import Specifiable
-from rlgraph.environments import SequentialVectorEnv
 
 
 class Worker(Specifiable):
     """
     Generic worker to locally interact with simulator environments.
     """
-    def __init__(self, agent, env_spec=None, num_envs=1, frameskip=1, render=False,
+    def __init__(self, agent, env_spec=None, num_environments=1, frameskip=1, render=False,
                  worker_executes_exploration=True, exploration_epsilon=0.1, episode_finish_callback=None):
         """
         Args:
             agent (Agent): Agent to execute environment on.
             env_spec Optional[Union[callable, dict]]): Either an environment spec or a callable returning a new
                 environment.
-            num_envs (int): How many single Environments should be run in parallel in a SequentialVectorEnv.
+            num_environments (int): How many single Environments should be run in parallel in a SequentialVectorEnv.
             frameskip (int): How often actions are repeated after retrieving them from the agent.
                 This setting can be overwritten in the single calls to the different `execute_..` methods.
             render (bool): Whether to render the environment after each action.
@@ -45,14 +46,23 @@ class Worker(Specifiable):
             exploration_epsilon (Optional[float]): Epsilon to use if worker executes exploration.
         """
         super(Worker, self).__init__()
-        self.num_environments = num_envs
+        self.num_environments = num_environments
         self.logger = logging.getLogger(__name__)
-        if env_spec is not None:
+
+        # VectorEnv was passed in directly -> Use that one.
+        if isinstance(env_spec, VectorEnv):
+            self.vector_env = env_spec
+            self.num_environments = self.vector_env.num_environments
             self.env_ids = ["env_{}".format(i) for i in range_(self.num_environments)]
-            self.vector_env = SequentialVectorEnv(env_spec=env_spec, num_envs=self.num_environments)
+        # `Env_spec` is for single envs inside a SequentialVectorEnv.
+        elif env_spec is not None:
+            self.vector_env = SequentialVectorEnv(env_spec=env_spec, num_environments=self.num_environments)
+            self.env_ids = ["env_{}".format(i) for i in range_(self.num_environments)]
+        # No env_spec.
         else:
-            self.env_ids = []
             self.vector_env = None
+            self.env_ids = []
+
         self.agent = agent
         self.frameskip = frameskip
         self.render = render
@@ -181,7 +191,7 @@ class Worker(Specifiable):
             # Are we allowed to update?
             if self.agent.timesteps > self.steps_before_update and \
                     (self.agent.observe_spec["buffer_enabled"] is False or  # No update before some data in buffer
-                     self.agent.timesteps >= self.agent.observe_spec["buffer_size"]):
+                     int(self.agent.timesteps / self.num_environments) >= self.agent.observe_spec["buffer_size"]):
                 # Updating according to one update mode:
                 if self.update_mode == "time_steps" and self.agent.timesteps % self.update_interval == 0:
                     return self.execute_update()
@@ -246,9 +256,9 @@ class Worker(Specifiable):
             return self.agent.get_action(states=states, use_exploration=use_exploration,
                                          apply_preprocessing=apply_preprocessing, extra_returns=extra_returns)
 
-    def log_finished_episode(self, reward, duration, timesteps, env_num=0):
-        self.logger.debug("Finished episode: reward={}, actions={}, duration={}s.".format(
-            reward, duration, timesteps))
+    def log_finished_episode(self, episode_return, duration, timesteps, env_num=0):
+        self.logger.debug("Finished episode: return={}, actions={}, duration={}s.".format(
+            episode_return, duration, timesteps))
 
         if self.episode_finish_callback:
-            self.episode_finish_callback(reward=reward, duration=duration, timesteps=timesteps, env_num=env_num)
+            self.episode_finish_callback(episode_return=episode_return, duration=duration, timesteps=timesteps, env_num=env_num)
