@@ -27,12 +27,12 @@ from rlgraph import get_backend
 from rlgraph.components.component import Component
 from rlgraph.spaces import Space, Dict
 from rlgraph.spaces.space_utils import get_space_from_op, check_space_equivalence
-from rlgraph.utils.execution_util import define_by_run_flatten, define_by_run_split_args, define_by_run_unflatten, \
+from rlgraph.utils.define_by_run_ops import define_by_run_flatten, define_by_run_split_args, define_by_run_unflatten, \
     define_by_run_unpack
 from rlgraph.utils.input_parsing import parse_summary_spec
 from rlgraph.utils.op_records import FlattenedDataOp, DataOpRecord, DataOpRecordColumnIntoGraphFn, \
     DataOpRecordColumnIntoAPIMethod, DataOpRecordColumnFromGraphFn, DataOpRecordColumnFromAPIMethod, get_call_param_name
-from rlgraph.utils.ops import is_constant, ContainerDataOp, DataOpDict, flatten_op, TraceContext
+from rlgraph.utils.ops import is_constant, ContainerDataOp, DataOpDict, flatten_op, unflatten_op, TraceContext
 from rlgraph.utils.rlgraph_errors import RLGraphError, RLGraphBuildError
 from rlgraph.utils.specifiable import Specifiable
 from rlgraph.utils.util import force_list, force_tuple, get_shape
@@ -64,7 +64,9 @@ class GraphBuilder(Specifiable):
         # The name of this model. Our root-Component gets this name.
         self.logger = logging.getLogger(__name__)
         self.name = name
-        self.action_space = action_space
+        self.action_space = None
+        if action_space is not None:
+            self.action_space = Space.from_spec(action_space)
         self.summary_spec = parse_summary_spec(summary_spec)
         self.max_build_iterations = max_build_iterations
         # Set of components that have been analyzed on why they remain input-incomplete.
@@ -840,8 +842,13 @@ class GraphBuilder(Specifiable):
 
             # API returns a dict.
             if len(self.api[api_method_name][1]) > 0 and self.api[api_method_name][1][0].kwarg is not None:
-                fetch_dict[api_method_name] = {op_rec.kwarg: op_rec.op for op_rec in self.api[api_method_name][1] if
-                                               return_ops is None or op_rec.kwarg in return_ops}
+                for op_rec in self.api[api_method_name][1]:
+                    if return_ops is None or op_rec.kwarg in return_ops:
+                        if api_method_name not in fetch_dict:
+                            fetch_dict[api_method_name] = {}
+                        flat_ops = flatten_op(op_rec.op, mapping=lambda o: o.op if isinstance(o, DataOpRecord) else o)
+                        fetch_dict[api_method_name][op_rec.kwarg] = unflatten_op(flat_ops)
+
                 if return_ops is not None:
                     assert all(op in fetch_dict[api_method_name] for op in return_ops),\
                         "ERROR: Not all wanted return_ops ({}) are returned by API-method `api_method_call`!".format(
