@@ -17,11 +17,12 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import cv2
-import numpy as np
 import os
 import unittest
 
+import cv2
+import numpy as np
+
 from rlgraph.components.layers import GrayScale, ReShape, Multiply, Divide, Clip, ImageBinary, ImageResize, ImageCrop, \
     MovingStandardize
 from rlgraph.environments import OpenAIGymEnv
@@ -34,7 +35,7 @@ class TestPreprocessLayers(unittest.TestCase):
 
     def test_multiply(self):
         multiply = Multiply(factor=2.0)
-        test = ComponentTest(component=multiply, input_spaces=dict(preprocessing_inputs=FloatBox(
+        test = ComponentTest(component=multiply, input_spaces=dict(inputs=FloatBox(
             shape=(2, 1), add_batch_rank=True)
         ))
 
@@ -42,25 +43,25 @@ class TestPreprocessLayers(unittest.TestCase):
         # Batch=2
         input_ = np.array([[[1.0], [2.0]], [[3.0], [4.0]]])
         expected = np.array([[[2.0], [4.0]], [[6.0], [8.0]]])
-        test.test(("apply", input_), expected_outputs=expected)
+        test.test(("call", input_), expected_outputs=expected)
 
     def test_divide(self):
         divide = Divide(divisor=10.0)
-        test = ComponentTest(component=divide, input_spaces=dict(preprocessing_inputs=FloatBox(shape=(1, 2),
+        test = ComponentTest(component=divide, input_spaces=dict(inputs=FloatBox(shape=(1, 2),
                                                                                                add_batch_rank=False)))
 
         test.test("reset")
 
         input_ = np.array([[10.0, 100.0]])
         expected = np.array([[1.0, 10.0]])
-        test.test(("apply", input_), expected_outputs=expected)
+        test.test(("call", input_), expected_outputs=expected)
 
     def test_clip(self):
         clip = Clip(min=0.0, max=1.0)
         # Grayscale image of 2x2 size.
         test = ComponentTest(
             component=clip,
-            input_spaces=dict(preprocessing_inputs=FloatBox(shape=(2, 2), add_batch_rank=True))
+            input_spaces=dict(inputs=FloatBox(shape=(2, 2), add_batch_rank=True))
         )
 
         test.test("reset")
@@ -75,21 +76,21 @@ class TestPreprocessLayers(unittest.TestCase):
             [[0.0, 1.0], [0.0, 0.0]],
             [[0.0005, 0.00000009], [1.0, 1.0]]
         ])
-        test.test(("apply", input_images), expected_outputs=expected)
+        test.test(("call", input_images), expected_outputs=expected)
 
     def test_grayscale_with_uint8_image(self):
         # last rank is always the color rank (its dim must match len(grayscale-weights))
         space = IntBox(256, shape=(1, 1, 2), dtype="uint8", add_batch_rank=True)
         grayscale = GrayScale(weights=(0.5, 0.5), keep_rank=False)
 
-        test = ComponentTest(component=grayscale, input_spaces=dict(preprocessing_inputs=space))
+        test = ComponentTest(component=grayscale, input_spaces=dict(inputs=space))
 
         # Run the test (batch of 3 images).
         input_ = space.sample(size=3)
         expected = np.sum(input_, axis=-1, keepdims=False)
         expected = (expected / 2).astype(input_.dtype)
         test.test("reset")
-        print(test.test(("apply", input_), expected_outputs=expected))
+        print(test.test(("call", input_), expected_outputs=expected))
 
     def test_grayscale_python_with_uint8_image(self):
         # last rank is always the color rank (its dim must match len(grayscale-weights))
@@ -100,7 +101,7 @@ class TestPreprocessLayers(unittest.TestCase):
         input_ = space.sample(size=2)
         expected = np.round(np.dot(input_[:, :, :, :3], [0.299, 0.587, 0.114]), 0).astype(dtype=input_.dtype)
 
-        out = grayscale._graph_fn_apply(input_)
+        out = grayscale._graph_fn_call(input_)
         recursive_assert_almost_equal(out, expected)
 
     def test_split_inputs_on_grayscale(self):
@@ -112,7 +113,7 @@ class TestPreprocessLayers(unittest.TestCase):
         ))
         grayscale = GrayScale(weights=(0.5, 0.5), keep_rank=False)
 
-        test = ComponentTest(component=grayscale, input_spaces=dict(preprocessing_inputs=space))
+        test = ComponentTest(component=grayscale, input_spaces=dict(inputs=space))
 
         # Run the test.
         input_ = dict(
@@ -134,7 +135,7 @@ class TestPreprocessLayers(unittest.TestCase):
             c=0.7
         )
         test.test("reset")
-        test.test(("apply", input_), expected_outputs=expected)
+        test.test(("call", input_), expected_outputs=expected)
 
     def test_split_graph_on_reshape_flatten(self):
         space = Dict.from_spec(
@@ -148,7 +149,7 @@ class TestPreprocessLayers(unittest.TestCase):
         )
         flatten = ReShape(flatten=True, flatten_categories={"d": 3})
 
-        test = ComponentTest(component=flatten, input_spaces=dict(preprocessing_inputs=space))
+        test = ComponentTest(component=flatten, input_spaces=dict(inputs=space))
 
         input_ = dict(
             a=(
@@ -171,13 +172,13 @@ class TestPreprocessLayers(unittest.TestCase):
             d=np.array([[0.0, 0.0, 1.0], [1.0, 0.0, 0.0]])  # category (one-hot) flatten
         )
         test.test("reset")
-        test.test(("apply", input_), expected_outputs=expected)
+        test.test(("call", input_), expected_outputs=expected)
 
     def test_image_resize(self):
         image_resize = ImageResize(width=4, height=4, interpolation="bilinear")
         # Some image of 16x16x3 size.
         test = ComponentTest(
-            component=image_resize, input_spaces=dict(preprocessing_inputs=FloatBox(shape=(16, 16, 3), add_batch_rank=False))
+            component=image_resize, input_spaces=dict(inputs=FloatBox(shape=(16, 16, 3), add_batch_rank=False))
         )
 
         test.test("reset")
@@ -186,14 +187,14 @@ class TestPreprocessLayers(unittest.TestCase):
         expected = cv2.imread(os.path.join(os.path.dirname(__file__), "images/4x4x3_image_resized.bmp"))
         assert expected is not None
 
-        test.test(("apply", input_image), expected_outputs=expected)
+        test.test(("call", input_image), expected_outputs=expected)
 
     def test_image_crop(self):
         image_crop = ImageCrop(x=7, y=1, width=8, height=12)
 
         # Some image of 16x16x3 size.
         test = ComponentTest(
-            component=image_crop, input_spaces=dict(preprocessing_inputs=FloatBox(shape=(16, 16, 3),
+            component=image_crop, input_spaces=dict(inputs=FloatBox(shape=(16, 16, 3),
                                                                                   add_batch_rank=False))
         )
 
@@ -203,25 +204,25 @@ class TestPreprocessLayers(unittest.TestCase):
         expected = cv2.imread(os.path.join(os.path.dirname(__file__), "images/8x12x3_image_cropped.bmp"))
         assert expected is not None
 
-        test.test(("apply", input_image), expected_outputs=expected)
+        test.test(("call", input_image), expected_outputs=expected)
 
     def test_python_image_crop(self):
         image_crop = ImageCrop(x=7, y=1, width=8, height=12, backend="python")
         image_crop.create_variables(input_spaces=dict(
-            preprocessing_inputs=FloatBox(shape=(16, 16, 3)), add_batch_rank=False)
+            inputs=FloatBox(shape=(16, 16, 3)), add_batch_rank=False)
         )
 
         input_image = cv2.imread(os.path.join(os.path.dirname(__file__), "images/16x16x3_image.bmp"))
         expected = cv2.imread(os.path.join(os.path.dirname(__file__), "images/8x12x3_image_cropped.bmp"))
         assert expected is not None
 
-        out = image_crop._graph_fn_apply(input_image)
+        out = image_crop._graph_fn_call(input_image)
         recursive_assert_almost_equal(out, expected)
 
     def test_black_and_white(self):
         binary = ImageBinary()
         # Color image of 2x2x3 size.
-        test = ComponentTest(component=binary, input_spaces=dict(preprocessing_inputs=FloatBox(shape=(2, 2, 3), add_batch_rank=True)))
+        test = ComponentTest(component=binary, input_spaces=dict(inputs=FloatBox(shape=(2, 2, 3), add_batch_rank=True)))
 
         test.test("reset")
         # Batch=2
@@ -233,7 +234,7 @@ class TestPreprocessLayers(unittest.TestCase):
             [[1, 1], [0, 1]],
             [[1, 0], [0, 1]]
         ])
-        test.test(("apply", input_images), expected_outputs=expected)
+        test.test(("call", input_images), expected_outputs=expected)
 
     def test_moving_standardize_python(self):
         env = OpenAIGymEnv("Pong-v0")
@@ -241,12 +242,12 @@ class TestPreprocessLayers(unittest.TestCase):
 
         moving_standardize = MovingStandardize(backend="python")
         moving_standardize.create_variables(input_spaces=dict(
-                    preprocessing_inputs=space
+                    inputs=space
                 ), action_space=None)
         samples = [space.sample() for _ in range(100)]
         out = None
         for sample in samples:
-            out = moving_standardize._graph_fn_apply(sample)
+            out = moving_standardize._graph_fn_call(sample)
 
         # Assert shape remains intact.
         expected_shape = (1, ) + space.shape
