@@ -17,12 +17,13 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import unittest
 from copy import deepcopy
+
 import numpy as np
 from six.moves import xrange as range_
-import unittest
 
-from rlgraph.agents import ApexAgent, DQNAgent
+from rlgraph.agents import ApexAgent
 from rlgraph.components.layers import GrayScale, Multiply
 from rlgraph.components.neural_networks import PreprocessorStack
 from rlgraph.environments import SequentialVectorEnv
@@ -122,10 +123,10 @@ class TestPreprocessorStacks(unittest.TestCase):
             python_preprocessor = PreprocessorStack(*incremental_spec, backend="python")
             for sub_comp_scope in incremental_scopes:
                 python_preprocessor.sub_components[sub_comp_scope].create_variables(
-                    input_spaces=dict(preprocessing_inputs=in_space), action_space=None
+                    input_spaces=dict(inputs=in_space), action_space=None
                 )
                 python_preprocessor.sub_components[sub_comp_scope].check_input_spaces(
-                    input_spaces=dict(preprocessing_inputs=in_space), action_space=None
+                    input_spaces=dict(inputs=in_space), action_space=None
                 )
                 #build_space = python_processor.sub_components[sub_comp_scope].get_preprocessed_space(build_space)
                 python_preprocessor.reset()
@@ -164,7 +165,7 @@ class TestPreprocessorStacks(unittest.TestCase):
         python_preprocessor = PreprocessorStack(*specs, backend="python")
         for sub_comp_scope in scopes:
             python_preprocessor.sub_components[sub_comp_scope].create_variables(
-                input_spaces=dict(preprocessing_inputs=in_space), action_space=None
+                input_spaces=dict(inputs=in_space), action_space=None
             )
             python_preprocessor.reset()
 
@@ -188,7 +189,7 @@ class TestPreprocessorStacks(unittest.TestCase):
             episodic_life=True
         )
         # Test with batching because we assume vector environments to be the normal case going forward.
-        env = SequentialVectorEnv(num_envs=4, env_spec=env_spec, num_background_envs=2)
+        env = SequentialVectorEnv(num_environments=4, env_spec=env_spec, num_background_envs=2)
         in_space = env.state_space
 
         agent_config = config_from_path("configs/ray_apex_for_pong.json")
@@ -201,7 +202,7 @@ class TestPreprocessorStacks(unittest.TestCase):
             spec["backend"] = "python"
         python_processor = PreprocessorStack(*preprocessing_spec, backend="python")
         for sub_comp_scope in scopes:
-            python_processor.sub_components[sub_comp_scope].create_variables(dict(preprocessing_inputs=in_space))
+            python_processor.sub_components[sub_comp_scope].create_variables(dict(inputs=in_space))
         python_processor.reset()
 
         # To have the use case we considered so far, use agent interface for TF backend.
