@@ -75,6 +75,31 @@ class TestApexExecutor(unittest.TestCase):
                 "ERROR: state '{}' not expected in q-table as it's a terminal state!".format(state)
             recursive_assert_almost_equal(q_values, expected_q_values_per_state[state], decimals=0)
 
+    def test_learning_2x2_grid_world_container_actions(self):
+        """
+        Tests Apex container action functionality.
+        """
+        env_spec = dict(
+            type="grid-world",
+            world="2x2",
+            save_mode=False,
+            action_type="ftj",
+            state_representation="xy+orientation"
+        )
+        agent_config = config_from_path("configs/apex_agent_for_2x2_gridworld_with_container_actions.json")
+        executor = ApexExecutor(
+            environment_spec=env_spec,
+            agent_config=agent_config,
+        )
+        # Define executor, test assembly.
+        print("Successfully created executor.")
+
+        # Executes actual workload.
+        result = executor.execute_workload(workload=dict(
+            num_timesteps=10000, report_interval=100, report_interval_min_seconds=1)
+        )
+        print(result)
+
     def test_learning_cartpole(self):
         """
         Tests if apex can learn a simple environment using a single worker, thus replicating
@@ -201,7 +226,7 @@ class TestApexExecutor(unittest.TestCase):
             build_space = in_space
             for sub_comp_scope in scopes:
                 processor_stack.sub_components[sub_comp_scope].create_variables(input_spaces=dict(
-                    preprocessing_inputs=build_space
+                    inputs=build_space
                 ), action_space=None)
                 build_space = processor_stack.sub_components[sub_comp_scope].get_preprocessed_space(build_space)
             processor_stack.reset()
